# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - FetchMessages Action

    @copyright: 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinSupport import getMetadata, writeHeaders
from MoinMessage import Message
from MoinMessageSupport import MoinMessageAction
from email.mime.text import MIMEText
from email.parser import Parser
from itertools import islice

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

Dependencies = ['pages']

class FetchMessages(MoinMessageAction):

    "A handler for requests accessing messages."

    def handle_message_content(self, content):

        "Handle the given message 'content'."

        request = self.request

        # NOTE: Could employ a more accurate content type.

        if not content.get_content_type() == "text/plain":
            writeHeaders(request, "text/plain", getMetadata(self.page), "403 Forbidden")
            request.write("The content does not appear to be a request for messages.")
            return

        # Obtain commands from the payload, returning a collection of messages.

        commands = content.get_payload(decode=True)

        # Build a container for the responses.

        message = Message()

        # Process each command, using RFC 1939 (POP3) as inspiration.

        for command in commands.split("\n"):
            command = command.strip()

            # Get the command and arguments.

            command_parts = command.split(None, 1)
            cmd = command_parts[0]

            # A request to count the messages is returned in a part.

            if cmd == "STAT":
                result = str(len(self.store))
                part = MIMEText(result, "plain")
                part["Request-Type"] = "STAT"
                part["Request-Status"] = "OK"
                message.add_update(part)

            # A request for specific messages returns each message in its own
            # part.

            elif cmd in ("RETR", "DELE"):

                try:
                    # Either select all.

                    if len(command_parts) == 1:
                        count = None

                    # Or select a particular number.

                    else:
                        count = int(parameters[1])

                except ValueError:
                    part = MIMEText(command, "plain")
                    part["Request-Type"] = cmd
                    part["Request-Status"] = "ERR"
                    message.add_update(part)

                else:
                    # A request for specific messages returns each message
                    # in its own part within a collection part.

                    if cmd == "RETR":
                        container = Message()

                        for message_text in islice(iter(self.store), count):
                            message_item = Parser().parse(StringIO(message_text))
                            container.add_update(message_item)

                        # Convert the container to a proper multipart section.

                        message.add_update(container.get_payload())

                    # A request to delete messages is performed immediately.

                    elif cmd == "DELE":
                        keys = self.store.keys()[:count]
                        keys.sort()

                        for key in keys:
                            del self.store[key]

                        part = MIMEText(result, "plain")
                        part["Request-Type"] = cmd
                        part["Request-Status"] = "OK"
                        message.add_update(part)

            # Handle invalid commands.

            elif cmd:
                part = MIMEText(result, "plain")
                part["Request-Type"] = cmd
                part["Request-Status"] = "ERR"
                message.add_update(part)

        # Write the response.

        request.write(message.get_payload().as_string())
        return 1, None

# Action function.

def execute(pagename, request):
    FetchMessages(pagename, request).do_action() # instead of render

# vim: tabstop=4 expandtab shiftwidth=4
